@@ -1,20 +1,22 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+import copy
 import importlib
 import logging
-
 import re
-from saml2.saml import NAME_FORMAT_URI
 import six
-from saml2 import xmlenc
 
 from saml2 import saml
-
-from saml2.time_util import instant, in_a_while
-from saml2.attribute_converter import from_local, get_local_name
-from saml2.s_utils import sid, MissingValue
-from saml2.s_utils import factory
+from saml2 import xmlenc
+from saml2.attribute_converter import from_local, ac_factory
+from saml2.attribute_converter import get_local_name
 from saml2.s_utils import assertion_factory
+from saml2.s_utils import factory
+from saml2.s_utils import sid
+from saml2.s_utils import MissingValue
+from saml2.saml import NAME_FORMAT_URI
+from saml2.time_util import instant
+from saml2.time_util import in_a_while
 
 logger = logging.getLogger(__name__)
 
@@ -79,17 +81,29 @@ def filter_on_attributes(ava, required=None, optional=None, acs=None,
     """
 
     def _match_attr_name(attr, ava):
-        try:
-            friendly_name = attr["friendly_name"]
-        except KeyError:
-            friendly_name = get_local_name(acs, attr["name"], attr["name_format"])
+        local_name = None
 
-        _fn = _match(friendly_name, ava)
-        if not _fn:  # In the unlikely case that someone has provided us with URIs as attribute names
+        for a in ['name_format', 'friendly_name']:
+            _val = attr.get(a)
+            if _val:
+                if a == 'name_format':
+                    local_name = get_local_name(acs, attr['name'], _val)
+                else:
+                    local_name = _val
+                break
+
+        if local_name:
+            _fn = _match(local_name, ava)
+        else:
+            _fn = None
+
+        if not _fn:  # In the unlikely case that someone has provided us with
+            #  URIs as attribute names
             _fn = _match(attr["name"], ava)
 
         return _fn
 
+
     def _apply_attr_value_restrictions(attr, res, must=False):
         try:
             values = [av["text"] for av in attr["attribute_value"]]
@@ -104,7 +118,6 @@ def filter_on_attributes(ava, required=None, optional=None, acs=None,
         return _filter_values(ava[_fn], values, must)
 
     res = {}
-
     if required is None:
         required = []
 
@@ -114,8 +127,7 @@ def filter_on_attributes(ava, required=None, optional=None, acs=None,
         if _fn:
             _apply_attr_value_restrictions(attr, res, True)
         elif fail_on_unfulfilled_requirements:
-            desc = "Required attribute missing: '%s' (%s)" % (attr["name"],
-                                                              _fn)
+            desc = "Required attribute missing: '%s'" % (attr["name"])
             raise MissingValue(desc)
 
     if optional is None:
@@ -152,8 +164,8 @@ def filter_on_demands(ava, required=None, optional=None):
                 for val in vals:
                     if val not in ava[lava[attr]]:
                         raise MissingValue(
-                                "Required attribute value missing: %s,%s" % (attr,
-                                                                             val))
+                            "Required attribute value missing: %s,%s" % (attr,
+                                                                         val))
         else:
             raise MissingValue("Required attribute missing: %s" % (attr,))
 
@@ -266,20 +278,23 @@ def restriction_from_attribute_spec(attributes):
 
 def post_entity_categories(maps, **kwargs):
     restrictions = {}
+    try:
+        required = [d['friendly_name'].lower() for d in kwargs['required']]
+    except (KeyError, TypeError):
+        required = []
+
     if kwargs["mds"]:
-        try:
+        if "sp_entity_id" in kwargs:
             ecs = kwargs["mds"].entity_categories(kwargs["sp_entity_id"])
-        except KeyError:
             for ec_map in maps:
-                for attr in ec_map[""]:
-                    restrictions[attr] = None
-        else:
-            for ec_map in maps:
-                for key, val in ec_map.items():
+                for key, (atlist, only_required) in ec_map.items():
                     if key == "":  # always released
-                        attrs = val
+                        attrs = atlist
                     elif isinstance(key, tuple):
-                        attrs = val
+                        if only_required:
+                            attrs = [a for a in atlist if a in required]
+                        else:
+                            attrs = atlist
                         for _key in key:
                             try:
                                 assert _key in ecs
@@ -287,12 +302,19 @@ def post_entity_categories(maps, **kwargs):
                                 attrs = []
                                 break
                     elif key in ecs:
-                        attrs = val
+                        if only_required:
+                            attrs = [a for a in atlist if a in required]
+                        else:
+                            attrs = atlist
                     else:
                         attrs = []
 
                     for attr in attrs:
                         restrictions[attr] = None
+        else:
+            for ec_map in maps:
+                for attr in ec_map[""]:
+                    restrictions[attr] = None
 
     return restrictions
 
@@ -319,7 +341,7 @@ class Policy(object):
             a compiled regular expression.
         """
 
-        self._restrictions = restrictions.copy()
+        self._restrictions = copy.deepcopy(restrictions)
 
         for who, spec in self._restrictions.items():
             if spec is None:
@@ -331,11 +353,19 @@ class Policy(object):
             else:
                 ecs = []
                 for cat in items:
-                    _mod = importlib.import_module(
+                    try:
+                        _mod = importlib.import_module(cat)
+                    except ImportError:
+                        _mod = importlib.import_module(
                             "saml2.entity_category.%s" % cat)
                     _ec = {}
                     for key, items in _mod.RELEASE.items():
-                        _ec[key] = [k.lower() for k in items]
+                        alist = [k.lower() for k in items]
+                        try:
+                            _only_required = _mod.ONLY_REQUIRED[key]
+                        except (AttributeError, KeyError):
+                            _only_required = False
+                        _ec[key] = (alist, _only_required)
                     ecs.append(_ec)
                 spec["entity_categories"] = ecs
             try:
@@ -444,7 +474,7 @@ class Policy(object):
                 pass
         return []
 
-    def get_entity_categories(self, sp_entity_id, mds):
+    def get_entity_categories(self, sp_entity_id, mds, required):
         """
 
         :param sp_entity_id:
@@ -452,7 +482,7 @@ class Policy(object):
         :return: A dictionary with restrictions
         """
 
-        kwargs = {"mds": mds}
+        kwargs = {"mds": mds, 'required': required}
 
         return self.get("entity_categories", sp_entity_id, default={},
                         post_func=post_entity_categories, **kwargs)
@@ -483,19 +513,18 @@ class Policy(object):
         """
 
         _ava = None
-        if required or optional:
+
+        if not self.acs:  # acs MUST have a value, fall back to default.
+            self.acs = ac_factory()
+
+        _rest = self.get_entity_categories(sp_entity_id, mdstore, required)
+        if _rest:
+            _ava = filter_attribute_value_assertions(ava.copy(), _rest)
+        elif required or optional:
             logger.debug("required: %s, optional: %s", required, optional)
             _ava = filter_on_attributes(
-                    ava.copy(), required, optional, self.acs,
-                    self.get_fail_on_missing_requested(sp_entity_id))
-
-        _rest = self.get_entity_categories(sp_entity_id, mdstore)
-        if _rest:
-            ava_ec = filter_attribute_value_assertions(ava.copy(), _rest)
-            if _ava is None:
-                _ava = ava_ec
-            else:
-                _ava.update(ava_ec)
+                ava.copy(), required, optional, self.acs,
+                self.get_fail_on_missing_requested(sp_entity_id))
 
         _rest = self.get_attribute_restrictions(sp_entity_id)
         if _rest:
@@ -537,9 +566,9 @@ class Policy(object):
                        # How long might depend on who's getting it
                        not_on_or_after=self.not_on_or_after(sp_entity_id),
                        audience_restriction=[factory(
-                               saml.AudienceRestriction,
-                               audience=[factory(saml.Audience,
-                                                 text=sp_entity_id)])])
+                           saml.AudienceRestriction,
+                           audience=[factory(saml.Audience,
+                                             text=sp_entity_id)])])
 
     def get_sign(self, sp_entity_id):
         """
@@ -569,7 +598,7 @@ def _authn_context_class_ref(authn_class, authn_auth=None):
         return factory(saml.AuthnContext,
                        authn_context_class_ref=cntx_class,
                        authenticating_authority=factory(
-                               saml.AuthenticatingAuthority, text=authn_auth))
+                           saml.AuthenticatingAuthority, text=authn_auth))
     else:
         return factory(saml.AuthnContext,
                        authn_context_class_ref=cntx_class)
@@ -585,7 +614,7 @@ def _authn_context_decl(decl, authn_auth=None):
     return factory(saml.AuthnContext,
                    authn_context_decl=decl,
                    authenticating_authority=factory(
-                           saml.AuthenticatingAuthority, text=authn_auth))
+                       saml.AuthenticatingAuthority, text=authn_auth))
 
 
 def _authn_context_decl_ref(decl_ref, authn_auth=None):
@@ -598,12 +627,12 @@ def _authn_context_decl_ref(decl_ref, authn_auth=None):
     return factory(saml.AuthnContext,
                    authn_context_decl_ref=decl_ref,
                    authenticating_authority=factory(
-                           saml.AuthenticatingAuthority, text=authn_auth))
+                       saml.AuthenticatingAuthority, text=authn_auth))
 
 
 def authn_statement(authn_class=None, authn_auth=None,
                     authn_decl=None, authn_decl_ref=None, authn_instant="",
-                    subject_locality=""):
+                    subject_locality="", session_not_on_or_after=None):
     """
     Construct the AuthnStatement
     :param authn_class: Authentication Context Class reference
@@ -624,29 +653,33 @@ def authn_statement(authn_class=None, authn_auth=None,
 
     if authn_class:
         res = factory(
-                saml.AuthnStatement,
-                authn_instant=_instant,
-                session_index=sid(),
-                authn_context=_authn_context_class_ref(
-                        authn_class, authn_auth))
+            saml.AuthnStatement,
+            authn_instant=_instant,
+            session_index=sid(),
+            session_not_on_or_after=session_not_on_or_after,
+            authn_context=_authn_context_class_ref(
+                authn_class, authn_auth))
     elif authn_decl:
         res = factory(
-                saml.AuthnStatement,
-                authn_instant=_instant,
-                session_index=sid(),
-                authn_context=_authn_context_decl(authn_decl, authn_auth))
+            saml.AuthnStatement,
+            authn_instant=_instant,
+            session_index=sid(),
+            session_not_on_or_after=session_not_on_or_after,
+            authn_context=_authn_context_decl(authn_decl, authn_auth))
     elif authn_decl_ref:
         res = factory(
-                saml.AuthnStatement,
-                authn_instant=_instant,
-                session_index=sid(),
-                authn_context=_authn_context_decl_ref(authn_decl_ref,
-                                                      authn_auth))
+            saml.AuthnStatement,
+            authn_instant=_instant,
+            session_index=sid(),
+            session_not_on_or_after=session_not_on_or_after,
+            authn_context=_authn_context_decl_ref(authn_decl_ref,
+                                                  authn_auth))
     else:
         res = factory(
-                saml.AuthnStatement,
-                authn_instant=_instant,
-                session_index=sid())
+            saml.AuthnStatement,
+            authn_instant=_instant,
+            session_index=sid(),
+            session_not_on_or_after=session_not_on_or_after)
 
     if subject_locality:
         res.subject_locality = saml.SubjectLocality(text=subject_locality)
@@ -654,6 +687,48 @@ def authn_statement(authn_class=None, authn_auth=None,
     return res
 
 
+def do_subject_confirmation(policy, sp_entity_id, key_info=None, **treeargs):
+    """
+
+    :param policy: Policy instance
+    :param sp_entity_id: The entityid of the SP
+    :param subject_confirmation_method: How was the subject confirmed
+    :param address: The network address/location from which an attesting entity
+        can present the assertion.
+    :param key_info: Information of the key used to confirm the subject
+    :param in_response_to: The ID of a SAML protocol message in response to
+        which an attesting entity can present the assertion.
+    :param recipient: A URI specifying the entity or location to which an
+        attesting entity can present the assertion.
+    :param not_before: A time instant before which the subject cannot be
+        confirmed. The time value MUST be encoded in UTC.
+    :return:
+    """
+
+    _sc = factory(saml.SubjectConfirmation, **treeargs)
+
+    _scd = _sc.subject_confirmation_data
+    _scd.not_on_or_after = policy.not_on_or_after(sp_entity_id)
+
+    if _sc.method == saml.SCM_HOLDER_OF_KEY:
+        _scd.add_extension_element(key_info)
+
+    return _sc
+
+
+def do_subject(policy, sp_entity_id, name_id, **farg):
+    #
+    specs = farg['subject_confirmation']
+
+    if isinstance(specs, list):
+        res = [do_subject_confirmation(policy, sp_entity_id, **s) for s in
+               specs]
+    else:
+        res = [do_subject_confirmation(policy, sp_entity_id, **specs)]
+
+    return factory(saml.Subject, name_id=name_id, subject_confirmation=res)
+
+
 class Assertion(dict):
     """ Handles assertions about subjects """
 
@@ -661,17 +736,16 @@ class Assertion(dict):
         dict.__init__(self, dic)
         self.acs = []
 
-    def construct(self, sp_entity_id, in_response_to, consumer_url,
-                  name_id, attrconvs, policy, issuer, authn_class=None,
-                  authn_auth=None, authn_decl=None, encrypt=None,
-                  sec_context=None, authn_decl_ref=None, authn_instant="",
-                  subject_locality="", authn_statem=None, add_subject=True):
+    def construct(self, sp_entity_id, attrconvs, policy, issuer, farg,
+                  authn_class=None, authn_auth=None, authn_decl=None,
+                  encrypt=None, sec_context=None, authn_decl_ref=None,
+                  authn_instant="", subject_locality="", authn_statem=None,
+                  name_id=None, session_not_on_or_after=None):
         """ Construct the Assertion
 
         :param sp_entity_id: The entityid of the SP
         :param in_response_to: An identifier of the message, this message is
             a response to
-        :param consumer_url: The intended consumer of the assertion
         :param name_id: An NameID instance
         :param attrconvs: AttributeConverters
         :param policy: The policy that should be adhered to when replying
@@ -696,7 +770,7 @@ class Assertion(dict):
             _name_format = NAME_FORMAT_URI
 
         attr_statement = saml.AttributeStatement(attribute=from_local(
-                attrconvs, self, _name_format))
+            attrconvs, self, _name_format))
 
         if encrypt == "attributes":
             for attr in attr_statement.attribute:
@@ -717,33 +791,16 @@ class Assertion(dict):
             _authn_statement = authn_statement(authn_class, authn_auth,
                                                authn_decl, authn_decl_ref,
                                                authn_instant,
-                                               subject_locality)
+                                               subject_locality,
+                                               session_not_on_or_after=session_not_on_or_after)
         else:
             _authn_statement = None
 
-        if not add_subject:
-            _ass = assertion_factory(
-                    issuer=issuer,
-                    conditions=conds,
-                    subject=None
-            )
-        else:
-            _ass = assertion_factory(
-                    issuer=issuer,
-                    conditions=conds,
-                    subject=factory(
-                            saml.Subject,
-                            name_id=name_id,
-                            subject_confirmation=[factory(
-                                    saml.SubjectConfirmation,
-                                    method=saml.SCM_BEARER,
-                                    subject_confirmation_data=factory(
-                                            saml.SubjectConfirmationData,
-                                            in_response_to=in_response_to,
-                                            recipient=consumer_url,
-                                            not_on_or_after=policy.not_on_or_after(sp_entity_id)))]
-                    ),
-            )
+        subject = do_subject(policy, sp_entity_id, name_id,
+                             **farg['subject'])
+
+        _ass = assertion_factory(issuer=issuer, conditions=conds,
+                                 subject=subject)
 
         if _authn_statement:
             _ass.authn_statement = [_authn_statement]
