@@ -31,11 +31,6 @@ from saml2.client_base import LogoutError
 from saml2.client_base import NoServiceDefined
 from saml2.mdstore import destinations
 
-try:
-    from urllib.parse import parse_qs
-except ImportError:
-    from urlparse import parse_qs
-
 import logging
 
 logger = logging.getLogger(__name__)
@@ -68,17 +63,17 @@ class Saml2Client(Base):
 
         reqid, negotiated_binding, info = \
             self.prepare_for_negotiated_authenticate(
-            entityid=entityid,
-            relay_state=relay_state,
-            binding=binding,
-            vorg=vorg,
-            nameid_format=nameid_format,
-            scoping=scoping,
-            consent=consent,
-            extensions=extensions,
-            sign=sign,
-            response_binding=response_binding,
-            **kwargs)
+                entityid=entityid,
+                relay_state=relay_state,
+                binding=binding,
+                vorg=vorg,
+                nameid_format=nameid_format,
+                scoping=scoping,
+                consent=consent,
+                extensions=extensions,
+                sign=sign,
+                response_binding=response_binding,
+                **kwargs)
 
         assert negotiated_binding == binding
 
@@ -89,8 +84,7 @@ class Saml2Client(Base):
             nameid_format=None, scoping=None, consent=None, extensions=None,
             sign=None, response_binding=saml2.BINDING_HTTP_POST, **kwargs):
         """ Makes all necessary preparations for an authentication request
-        that negotiates
-        which binding to use for authentication.
+        that negotiates which binding to use for authentication.
 
         :param entityid: The entity ID of the IdP to send the request to
         :param relay_state: To where the user should be returned after
@@ -126,12 +120,12 @@ class Saml2Client(Base):
             logger.info("AuthNReq: %s", _req_str)
 
             try:
-                sigalg = kwargs["sigalg"]
+                args = {'sigalg': kwargs["sigalg"]}
             except KeyError:
-                sigalg = ""
+                args = {}
 
             http_info = self.apply_binding(binding, _req_str, destination,
-                                           relay_state, sigalg=sigalg)
+                                           relay_state, sign=sign, **args)
 
             return reqid, binding, http_info
         else:
@@ -235,7 +229,7 @@ class Saml2Client(Base):
                     if binding == BINDING_HTTP_REDIRECT:
                         sigalg = kwargs.get(
                             "sigalg", ds.DefaultSignature().get_sign_alg())
-                        #key = kwargs.get("key", self.signkey)
+                        # key = kwargs.get("key", self.signkey)
                         srequest = str(request)
                     else:
                         srequest = self.sign(request, sign_alg=sign_alg,
@@ -246,7 +240,7 @@ class Saml2Client(Base):
                 relay_state = self._relay_state(req_id)
 
                 http_info = self.apply_binding(binding, srequest, destination,
-                                               relay_state, sigalg=sigalg)
+                                               relay_state, sign=sign, sigalg=sigalg)
 
                 if binding == BINDING_SOAP:
                     response = self.send(**http_info)
@@ -267,7 +261,7 @@ class Saml2Client(Base):
                                           "entity_ids": entity_ids,
                                           "name_id": code(name_id),
                                           "reason": reason,
-                                          "not_on_of_after": expire,
+                                          "not_on_or_after": expire,
                                           "sign": sign}
 
                     responses[entity_id] = (binding, http_info)
@@ -471,7 +465,7 @@ class Saml2Client(Base):
                                   attribute=attribute,
                                   sp_name_qualifier=sp_name_qualifier,
                                   name_qualifier=name_qualifier,
-                                  nameid_format=nameid_format,
+                                  format=nameid_format,
                                   response_args=response_args)
         elif binding == BINDING_HTTP_POST:
             mid = sid()
@@ -484,12 +478,12 @@ class Saml2Client(Base):
                                     "sign": sign}
             relay_state = self._relay_state(query.id)
             return self.apply_binding(binding, "%s" % query, destination,
-                                      relay_state)
+                                      relay_state, sign=sign)
         else:
             raise SAMLError("Unsupported binding")
 
     def handle_logout_request(self, request, name_id, binding, sign=False,
-                              relay_state=""):
+                              sign_alg=None, relay_state=""):
         """
         Deal with a LogoutRequest
 
@@ -536,9 +530,9 @@ class Saml2Client(Base):
                 "single_logout_service"]
 
         response = self.create_logout_response(_req.message, response_bindings,
-                                               status, sign)
+                                               status, sign, sign_alg=sign_alg)
         rinfo = self.response_args(_req.message, response_bindings)
 
         return self.apply_binding(rinfo["binding"], response,
                                   rinfo["destination"], relay_state,
-                                  response=True)
+                                  response=True, sign=sign)
