@@ -1,14 +1,14 @@
 import base64
-# from binascii import hexlify
-from binascii import hexlify
 import copy
 import logging
-from hashlib import sha1
-from Crypto.PublicKey import RSA
 import requests
 import six
+
+from binascii import hexlify
+from hashlib import sha1
+
 from saml2.metadata import ENDPOINTS
-from saml2.profile import paos, ecp
+from saml2.profile import paos, ecp, samlec
 from saml2.soap import parse_soap_enveloped_saml_artifact_resolve
 from saml2.soap import class_instances_from_soap_enveloped_saml_thingies
 from saml2.soap import open_soap_envelope
@@ -30,6 +30,7 @@ from saml2.saml import NameID
 from saml2.saml import EncryptedAssertion
 from saml2.saml import Issuer
 from saml2.saml import NAMEID_FORMAT_ENTITY
+from saml2.response import AuthnResponse
 from saml2.response import LogoutResponse
 from saml2.response import UnsolicitedResponse
 from saml2.time_util import instant
@@ -63,7 +64,7 @@ from saml2.httpbase import HTTPBase
 from saml2.sigver import security_context
 from saml2.sigver import response_factory
 from saml2.sigver import SigverError
-from saml2.sigver import CryptoBackendXmlSec1
+from saml2.sigver import SignatureError
 from saml2.sigver import make_temp
 from saml2.sigver import pre_encryption_part
 from saml2.sigver import pre_signature_part
@@ -224,8 +225,9 @@ class Entity(HTTPBase):
             info["method"] = "POST"
         elif binding == BINDING_HTTP_REDIRECT:
             logger.info("HTTP REDIRECT")
-            if 'sigalg' in kwargs:
-                signer = self.sec.sec_backend.get_signer(kwargs['sigalg'])
+            sigalg = kwargs.get("sigalg")
+            if sign and sigalg:
+                signer = self.sec.sec_backend.get_signer(sigalg)
             else:
                 signer = None
             info = self.use_http_get(msg_str, destination, relay_state, typ,
@@ -267,16 +269,8 @@ class Entity(HTTPBase):
             else:
                 descr_type = "spsso"
 
-        _url = _index = None
-        if request:
-            try:
-                _url = getattr(request, "%s_url" % service)
-            except AttributeError:
-                _url = None
-                try:
-                    _index = getattr(request, "%s_index" % service)
-                except AttributeError:
-                    pass
+        _url = getattr(request, "%s_url" % service, None)
+        _index = getattr(request, "%s_index" % service, None)
 
         for binding in bindings:
             try:
@@ -394,7 +388,7 @@ class Entity(HTTPBase):
                 else:
                     xmlstr = txt
             except Exception:
-                raise UnravelError()
+                raise UnravelError("Unravelling binding '%s' failed" % binding)
 
         return xmlstr
 
@@ -407,7 +401,8 @@ class Entity(HTTPBase):
         """
         return class_instances_from_soap_enveloped_saml_thingies(text, [paos,
                                                                         ecp,
-                                                                        samlp])
+                                                                        samlp,
+                                                                        samlec])
 
     @staticmethod
     def unpack_soap_message(text):
@@ -553,7 +548,6 @@ class Entity(HTTPBase):
         _certs = []
 
         if encrypt_cert:
-            _certs = []
             _certs.append(encrypt_cert)
         elif sp_entity_id is not None:
             _certs = self.metadata.certs(sp_entity_id, "any", "encryption")
@@ -601,7 +595,7 @@ class Entity(HTTPBase):
 
         :param in_response_to: The session identifier of the request
         :param consumer_url: The URL which should receive the response
-        :param status: The status of the response
+        :param status: An instance of samlp.Status
         :param issuer: The issuer of the response
         :param sign: Whether the response should be signed or not
         :param to_sign: If there are other parts to sign
@@ -1100,8 +1094,6 @@ class Entity(HTTPBase):
             otherwise the response.
         """
 
-        response = None
-
         if self.config.accepted_time_diff:
             kwargs["timeslack"] = self.config.accepted_time_diff
 
@@ -1111,67 +1103,114 @@ class Entity(HTTPBase):
             else:
                 kwargs["asynchop"] = True
 
-        if xmlstr:
-            if "return_addrs" not in kwargs:
-                if binding in [BINDING_HTTP_REDIRECT, BINDING_HTTP_POST]:
-                    try:
-                        # expected return address
-                        kwargs["return_addrs"] = self.config.endpoint(
-                            service, binding=binding)
-                    except Exception:
-                        logger.info("Not supposed to handle this!")
-                        return None
+        response = None
+        if not xmlstr:
+            return response
 
-            try:
-                response = response_cls(self.sec, **kwargs)
-            except Exception as exc:
-                logger.info("%s", exc)
-                raise
+        if "return_addrs" not in kwargs:
+            bindings = {
+                BINDING_SOAP,
+                BINDING_HTTP_REDIRECT,
+                BINDING_HTTP_POST,
+            }
+            if binding in bindings:
+                # expected return address
+                kwargs["return_addrs"] = self.config.endpoint(
+                        service,
+                        binding=binding,
+                        context=self.entity_type)
 
-            xmlstr = self.unravel(xmlstr, binding, response_cls.msgtype)
-            origxml = xmlstr
-            if not xmlstr:  # Not a valid reponse
-                return None
+        try:
+            response = response_cls(self.sec, **kwargs)
+        except Exception as exc:
+            logger.info("%s", exc)
+            raise
 
-            try:
-                response = response.loads(xmlstr, False, origxml=origxml)
-            except SigverError as err:
+        xmlstr = self.unravel(xmlstr, binding, response_cls.msgtype)
+        if not xmlstr:  # Not a valid reponse
+            return None
+
+        try:
+            response_is_signed = False
+            # Record the response signature requirement.
+            require_response_signature = response.require_response_signature
+            # Force the requirement that the response be signed in order to
+            # force signature checking to happen so that we can know whether
+            # or not the response is signed. The attribute on the response class
+            # is reset to the recorded value in the finally clause below.
+            response.require_response_signature = True
+            response = response.loads(xmlstr, False, origxml=xmlstr)
+        except SigverError as err:
+            if require_response_signature:
                 logger.error("Signature Error: %s", err)
                 raise
-            except UnsolicitedResponse:
-                logger.error("Unsolicited response")
-                raise
-            except Exception as err:
-                if "not well-formed" in "%s" % err:
-                    logger.error("Not well-formed XML")
-                raise
+            else:
+                # The response is not signed but a signature is not required
+                # so reset the attribute on the response class to the recorded
+                # value and attempt to consume the unpacked XML again.
+                response.require_response_signature = require_response_signature
+                response = response.loads(xmlstr, False, origxml=xmlstr)
+        except UnsolicitedResponse:
+            logger.error("Unsolicited response")
+            raise
+        except Exception as err:
+            if "not well-formed" in "%s" % err:
+                logger.error("Not well-formed XML")
+            raise
+        else:
+            response_is_signed = True
+        finally:
+            response.require_response_signature = require_response_signature
 
-            logger.debug("XMLSTR: %s", xmlstr)
+        logger.debug("XMLSTR: %s", xmlstr)
 
-            if response:
+        if not response:
+            return response
+
+        keys = None
+        if outstanding_certs:
+            try:
+                cert = outstanding_certs[response.in_response_to]
+            except KeyError:
                 keys = None
-                if outstanding_certs:
-                    try:
-                        cert = outstanding_certs[response.in_response_to]
-                    except KeyError:
-                        keys = None
-                    else:
-                        if not isinstance(cert, list):
-                            cert = [cert]
-                        keys = []
-                        for _cert in cert:
-                            keys.append(_cert["key"])
-                only_identity_in_encrypted_assertion = False
-                if "only_identity_in_encrypted_assertion" in kwargs:
-                    only_identity_in_encrypted_assertion = kwargs[
-                        "only_identity_in_encrypted_assertion"]
+            else:
+                if not isinstance(cert, list):
+                    cert = [cert]
+                keys = []
+                for _cert in cert:
+                    keys.append(_cert["key"])
 
+        try:
+            assertions_are_signed = False
+            # Record the assertions signature requirement.
+            require_signature = response.require_signature
+            # Force the requirement that the assertions be signed in order to
+            # force signature checking to happen so that we can know whether
+            # or not the assertions are signed. The attribute on the response class
+            # is reset to the recorded value in the finally clause below.
+            response.require_signature = True
+            # Verify that the assertion is syntactically correct and the
+            # signature on the assertion is correct if present.
+            response = response.verify(keys)
+        except SignatureError as err:
+            if require_signature:
+                logger.error("Signature Error: %s", err)
+                raise
+            else:
+                response.require_signature = require_signature
                 response = response.verify(keys)
+        else:
+            assertions_are_signed = True
+        finally:
+            response.require_signature = require_signature
 
-            if not response:
-                return None
-
-                # logger.debug(response)
+        # If so configured enforce that either the response is signed
+        # or the assertions within it are signed.
+        if response.require_signature_or_response_signature:
+            if not response_is_signed and not assertions_are_signed:
+                msg = "Neither the response nor the assertions are signed"
+                logger.error(msg)
+                raise SigverError(msg)
 
         return response
 
