@@ -1,15 +1,18 @@
 from __future__ import print_function
 import hashlib
+import importlib
+import json
 import logging
 import os
 import sys
-import json
-import requests
-import six
+
 from hashlib import sha1
 from os.path import isfile
 from os.path import join
 
+import requests
+import six
+
 from saml2 import md
 from saml2 import saml
 from saml2 import samlp
@@ -32,19 +35,37 @@ from saml2.validate import valid_instance
 from saml2.time_util import valid
 from saml2.validate import NotValid
 from saml2.sigver import security_context
-from importlib import import_module
+from saml2.extension.mdattr import NAMESPACE as NS_MDATTR
+from saml2.extension.mdattr import EntityAttributes
+from saml2.extension.mdui import NAMESPACE as NS_MDUI
+from saml2.extension.mdui import UIInfo
+from saml2.extension.mdui import DisplayName
+from saml2.extension.mdui import Description
+from saml2.extension.mdui import InformationURL
+from saml2.extension.mdui import PrivacyStatementURL
+from saml2.extension.mdui import Logo
 
-__author__ = 'rolandh'
 
 logger = logging.getLogger(__name__)
 
+classnames = {
+    "mdattr_entityattributes": "{ns}&{tag}".format(
+        ns=NS_MDATTR, tag=EntityAttributes.c_tag
+    ),
+    "mdui_uiinfo": "{ns}&{tag}".format(ns=NS_MDUI, tag=UIInfo.c_tag),
+    "mdui_uiinfo_display_name": "{ns}&{tag}".format(ns=NS_MDUI, tag=DisplayName.c_tag),
+    "mdui_uiinfo_description": "{ns}&{tag}".format(ns=NS_MDUI, tag=Description.c_tag),
+    "mdui_uiinfo_information_url": "{ns}&{tag}".format(
+        ns=NS_MDUI, tag=InformationURL.c_tag
+    ),
+    "mdui_uiinfo_privacy_statement_url": "{ns}&{tag}".format(
+        ns=NS_MDUI, tag=PrivacyStatementURL.c_tag
+    ),
+    "mdui_uiinfo_logo": "{ns}&{tag}".format(ns=NS_MDUI, tag=Logo.c_tag),
+}
 
-class ToOld(Exception):
-    pass
-
-
-class SourceNotFound(Exception):
-    pass
+ENTITY_CATEGORY = "http://macedir.org/entity-category"
+ENTITY_CATEGORY_SUPPORT = "http://macedir.org/entity-category-support"
 
 REQ2SRV = {
     # IDP
@@ -68,12 +89,14 @@ REQ2SRV = {
     "discovery_service_request": "discovery_response"
 }
 
-ENTITYATTRIBUTES = "urn:oasis:names:tc:SAML:metadata:attribute&EntityAttributes"
-ENTITY_CATEGORY = "http://macedir.org/entity-category"
-ENTITY_CATEGORY_SUPPORT = "http://macedir.org/entity-category-support"
+
+class ToOld(Exception):
+    pass
 
 
-# ---------------------------------------------------
+class SourceNotFound(Exception):
+    pass
+
 
 def load_extensions():
     from saml2 import extension
@@ -225,7 +248,7 @@ class MetaData(object):
         '''
         raise NotImplementedError
 
-    def load(self):
+    def load(self, *args, **kwargs):
         '''
         Loads the metadata
         '''
@@ -267,7 +290,7 @@ class MetaData(object):
         """
         Return any entity that matches the specification
 
-        :param typ:
+        :param typ: Type of entity
         :param service:
         :param binding:
         :return:
@@ -280,6 +303,37 @@ class MetaData(object):
 
         return res
 
+    def any2(self, typ, service, binding=None):
+        """
+
+        :param type:
+        :param service:
+        :param binding:
+        :return:
+        """
+        res = {}
+        for entid, item in self.items():
+            hit = False
+            try:
+                descr = item['{}sso_descriptor'.format(typ)]
+            except KeyError:
+                continue
+            else:
+                for desc in descr:
+                    try:
+                        srvs = desc[service]
+                    except KeyError:
+                        continue
+                    else:
+                        for srv in srvs:
+                            if srv['binding'] == binding:
+                                res[entid] = item
+                                hit = True
+                                break
+                    if hit:
+                        break
+        return res
+
     def bindings(self, entity_id, typ, service):
         """
         Get me all the bindings that are registered for a service entity
@@ -303,7 +357,7 @@ class MetaData(object):
         raise NotImplementedError
 
     def dumps(self):
-        return json.dumps(self.items(), indent=2)
+        return json.dumps(list(self.items()), indent=2)
 
     def with_descriptor(self, descriptor):
         '''
@@ -326,7 +380,7 @@ class MetaData(object):
         res = []
         if "extensions" in self[entity_id]:
             for elem in self[entity_id]["extensions"]["extension_elements"]:
-                if elem["__class__"] == ENTITYATTRIBUTES:
+                if elem["__class__"] == classnames["mdattr_entityattributes"]:
                     for attr in elem["attribute"]:
                         res.append(attr["text"])
 
@@ -438,7 +492,7 @@ class InMemoryMetaData(MetaData):
         if self.check_validity:
             try:
                 if not valid(entity_descr.valid_until):
-                    logger.error("Entity descriptor (entity id:%s) to old",
+                    logger.error("Entity descriptor (entity id:%s) too old",
                                  entity_descr.entity_id)
                     self.to_old.append(entity_descr.entity_id)
                     return
@@ -632,9 +686,10 @@ class MetaDataFile(InMemoryMetaData):
         self.cert = cert
 
     def get_metadata_content(self):
-        return open(self.filename, 'rb').read()
+        with open(self.filename, 'rb') as fp:
+            return fp.read()
 
-    def load(self):
+    def load(self, *args, **kwargs):
         _txt = self.get_metadata_content()
         return self.parse_and_check_signature(_txt)
 
@@ -655,13 +710,13 @@ class MetaDataLoader(MetaDataFile):
 
     @staticmethod
     def get_metadata_loader(func):
-        if callable(func):
+        if hasattr(func, '__call__'):
             return func
 
         i = func.rfind('.')
         module, attr = func[:i], func[i + 1:]
         try:
-            mod = import_module(module)
+            mod = importlib.import_module(module)
         except Exception as e:
             raise RuntimeError(
                 'Cannot find metadata provider function %s: "%s"' % (func, e))
@@ -673,7 +728,7 @@ class MetaDataLoader(MetaDataFile):
                 'Module "%s" does not define a "%s" metadata loader' % (
                     module, attr))
 
-        if not callable(metadata_loader):
+        if not hasattr(metadata_loader, '__call__'):
             raise RuntimeError(
                 'Metadata loader %s.%s must be callable' % (module, attr))
 
@@ -686,7 +741,7 @@ class MetaDataLoader(MetaDataFile):
 class MetaDataExtern(InMemoryMetaData):
     """
     Class that handles metadata store somewhere on the net.
-    Accessible but HTTP GET.
+    Accessible by HTTP GET.
     """
 
     def __init__(self, attrc, url=None, security=None, cert=None,
@@ -710,14 +765,14 @@ class MetaDataExtern(InMemoryMetaData):
         self.security = security
         self.http = http
 
-    def load(self):
+    def load(self, *args, **kwargs):
         """ Imports metadata by the use of HTTP GET.
         If the fingerprint is known the file will be checked for
         compliance before it is imported.
         """
         response = self.http.send(self.url)
         if response.status_code == 200:
-            _txt = response.text.encode("utf-8")
+            _txt = response.content
             return self.parse_and_check_signature(_txt)
         else:
             logger.info("Response status: %s", response.status_code)
@@ -734,8 +789,10 @@ class MetaDataMD(InMemoryMetaData):
         super(MetaDataMD, self).__init__(attrc, **kwargs)
         self.filename = filename
 
-    def load(self):
-        for key, item in json.loads(open(self.filename).read()):
+    def load(self, *args, **kwargs):
+        with open(self.filename) as fp:
+            data = json.load(fp)
+        for key, item in data:
             self.entity[key] = item
 
 
@@ -743,7 +800,10 @@ SAML_METADATA_CONTENT_TYPE = 'application/samlmetadata+xml'
 
 
 class MetaDataMDX(InMemoryMetaData):
-    """ Uses the md protocol to fetch entity information
+    """
+    Uses the MDQ protocol to fetch entity information.
+    The protocol is defined at:
+    https://datatracker.ietf.org/doc/draft-young-md-query-saml/
     """
 
     @staticmethod
@@ -751,25 +811,42 @@ class MetaDataMDX(InMemoryMetaData):
         return "{{sha1}}{}".format(
             hashlib.sha1(entity_id.encode("utf-8")).hexdigest())
 
-    def __init__(self, url, entity_transform=None):
+    def __init__(self, url=None, security=None, cert=None,
+                 entity_transform=None, **kwargs):
         """
         :params url: mdx service url
+        :params security: SecurityContext()
+        :params cert: certificate used to check signature of signed metadata
         :params entity_transform: function transforming (e.g. base64,
         sha1 hash or URL quote
         hash) the entity id. It is applied to the entity id before it is
         concatenated with the request URL sent to the MDX server. Defaults to
         sha1 transformation.
         """
-        super(MetaDataMDX, self).__init__(None, None)
-        self.url = url
+        super(MetaDataMDX, self).__init__(None, **kwargs)
+        if not url:
+            raise SAMLError('URL for MDQ server not specified.')
+
+        self.url = url.rstrip('/')
 
         if entity_transform:
             self.entity_transform = entity_transform
         else:
-
             self.entity_transform = MetaDataMDX.sha1_entity_transform
 
-    def load(self):
+        self.cert = cert
+        self.security = security
+
+        # We assume that the MDQ server will return a single entity
+        # described by a single <EntityDescriptor> element. The protocol
+        # does allow multiple entities to be returned in an
+        # <EntitiesDescriptor> element but we will not currently support
+        # that use case since it is unlikely to be leveraged for most
+        # flows.
+        self.node_name = "%s:%s" % (md.EntityDescriptor.c_namespace,
+                                      md.EntityDescriptor.c_tag)
+
+    def load(self, *args, **kwargs):
         # Do nothing
         pass
 
@@ -781,7 +858,7 @@ class MetaDataMDX(InMemoryMetaData):
             response = requests.get(mdx_url, headers={
                 'Accept': SAML_METADATA_CONTENT_TYPE})
             if response.status_code == 200:
-                _txt = response.text.encode("utf-8")
+                _txt = response.content
 
                 if self.parse_and_check_signature(_txt):
                     return self.entity[item]
@@ -807,7 +884,7 @@ class MetadataStore(MetaData):
         :params ca_certs:
         :params disable_ssl_certificate_validation:
         """
-        self.attrc = attrc
+        MetaData.__init__(self, attrc, check_validity=check_validity)
 
         if disable_ssl_certificate_validation:
             self.http = HTTPBase(verify=False, ca_bundle=ca_certs)
@@ -821,14 +898,15 @@ class MetadataStore(MetaData):
         self.filter = filter
         self.to_old = {}
 
-    def load(self, typ, *args, **kwargs):
+    def load(self, *args, **kwargs):
         if self.filter:
             _args = {"filter": self.filter}
         else:
             _args = {}
 
+        typ = args[0]
         if typ == "local":
-            key = args[0]
+            key = args[1]
             # if library read every file in the library
             if os.path.isdir(key):
                 files = [f for f in os.listdir(key) if isfile(join(key, f))]
@@ -845,8 +923,10 @@ class MetadataStore(MetaData):
             self.ii += 1
             key = self.ii
             kwargs.update(_args)
-            _md = InMemoryMetaData(self.attrc, args[0])
+            _md = InMemoryMetaData(self.attrc, args[1])
         elif typ == "remote":
+            if "url" not in kwargs:
+                raise ValueError("Remote metadata must be structured as a dict containing the key 'url'")
             key = kwargs["url"]
             for _key in ["node_name", "check_validity"]:
                 try:
@@ -861,11 +941,23 @@ class MetadataStore(MetaData):
                                  kwargs["url"], self.security,
                                  kwargs["cert"], self.http, **_args)
         elif typ == "mdfile":
-            key = args[0]
-            _md = MetaDataMD(self.attrc, args[0], **_args)
+            key = args[1]
+            _md = MetaDataMD(self.attrc, args[1], **_args)
         elif typ == "loader":
-            key = args[0]
-            _md = MetaDataLoader(self.attrc, args[0], **_args)
+            key = args[1]
+            _md = MetaDataLoader(self.attrc, args[1], **_args)
+        elif typ == "mdq":
+            if 'url' in kwargs:
+                key = kwargs['url']
+                url = kwargs['url']
+                cert = kwargs.get('cert')
+                security = self.security
+                entity_transform = kwargs.get('entity_transform', None)
+                _md = MetaDataMDX(url, security, cert, entity_transform)
+            else:
+                key = args[1]
+                url = args[1]
+                _md = MetaDataMDX(url)
         else:
             raise SAMLError("Unknown metadata type '%s'" % typ)
         _md.load()
@@ -891,7 +983,7 @@ class MetadataStore(MetaData):
                     raise SAMLError("Misconfiguration in metadata %s" % item)
                 mod, clas = key.rsplit('.', 1)
                 try:
-                    mod = import_module(mod)
+                    mod = importlib.import_module(mod)
                     MDloader = getattr(mod, clas)
                 except (ImportError, AttributeError):
                     raise SAMLError("Unknown metadata loader %s" % key)
@@ -956,10 +1048,7 @@ class MetadataStore(MetaData):
             try:
                 srvs = _md[entity_id][typ]
             except KeyError:
-                return None
-
-            if not srvs:
-                return srvs
+                continue
 
             res = []
             for srv in srvs:
@@ -969,6 +1058,8 @@ class MetadataStore(MetaData):
                             res.append(elem)
             return res
 
+        return None
+
     def ext_service(self, entity_id, typ, service, binding=None):
         known_entity = False
         for key, _md in self.metadata.items():
@@ -1173,7 +1264,7 @@ class MetadataStore(MetaData):
         except KeyError:
             return res
         for elem in ext["extension_elements"]:
-            if elem["__class__"] == ENTITYATTRIBUTES:
+            if elem["__class__"] == classnames["mdattr_entityattributes"]:
                 for attr in elem["attribute"]:
                     if attr["name"] not in res:
                         res[attr["name"]] = []
@@ -1181,6 +1272,113 @@ class MetadataStore(MetaData):
                         "attribute_value"]]
         return res
 
+    def _lookup_elements_by_cls(self, root, cls):
+        elements = (
+            element
+            for uiinfo in root
+            for element_key, elements in uiinfo.items()
+            if element_key != "__class__"
+            for element in elements
+            if element.get("__class__") == cls
+        )
+        return elements
+
+    def _lookup_elements_by_key(self, root, key):
+        elements = (
+            element
+            for uiinfo in root
+            for elements in [uiinfo.get(key, [])]
+            for element in elements
+        )
+        return elements
+
+    def mdui_uiinfo(self, entity_id):
+        try:
+            data = self[entity_id]
+        except KeyError:
+            data = {}
+
+        descriptor_names = (
+            item
+            for item in data.keys()
+            if item.endswith("_descriptor")
+        )
+        descriptors = (
+            descriptor
+            for descriptor_name in descriptor_names
+            for descriptor in self[entity_id].get(descriptor_name, [])
+        )
+        extensions = (
+            extension
+            for descriptor in descriptors
+            for extension in descriptor.get("extensions", {}).get("extension_elements", [])
+        )
+        uiinfos = (
+            extension
+            for extension in extensions
+            if extension.get("__class__") == classnames["mdui_uiinfo"]
+        )
+        return uiinfos
+
+    def _mdui_uiinfo_i18n_elements_lookup(self, entity_id, langpref, element_hint, lookup):
+        uiinfos = self.mdui_uiinfo(entity_id)
+        elements = lookup(uiinfos, element_hint)
+        lang_elements = (
+            element
+            for element in elements
+            if langpref is None or element.get("lang") == langpref
+        )
+        values = (
+            value
+            for element in lang_elements
+            for value in [element.get("text")]
+        )
+        return values
+
+    def mdui_uiinfo_i18n_element_cls(self, entity_id, langpref, element_cls):
+        values = self._mdui_uiinfo_i18n_elements_lookup(
+            entity_id, langpref, element_cls, self._lookup_elements_by_cls
+        )
+        return values
+
+    def mdui_uiinfo_i18n_element_key(self, entity_id, langpref, element_key):
+        values = self._mdui_uiinfo_i18n_elements_lookup(
+            entity_id, langpref, element_key, self._lookup_elements_by_key
+        )
+        return values
+
+    def mdui_uiinfo_display_name(self, entity_id, langpref=None):
+        cls = classnames["mdui_uiinfo_display_name"]
+        values = self.mdui_uiinfo_i18n_element_cls(entity_id, langpref, cls)
+        return values
+
+    def mdui_uiinfo_description(self, entity_id, langpref=None):
+        cls = classnames["mdui_uiinfo_description"]
+        values = self.mdui_uiinfo_i18n_element_cls(entity_id, langpref, cls)
+        return values
+
+    def mdui_uiinfo_information_url(self, entity_id, langpref=None):
+        cls = classnames["mdui_uiinfo_information_url"]
+        values = self.mdui_uiinfo_i18n_element_cls(entity_id, langpref, cls)
+        return values
+
+    def mdui_uiinfo_privacy_statement_url(self, entity_id, langpref=None):
+        cls = classnames["mdui_uiinfo_privacy_statement_url"]
+        values = self.mdui_uiinfo_i18n_element_cls(entity_id, langpref, cls)
+        return values
+
+    def mdui_uiinfo_logo(self, entity_id, width=None, height=None):
+        uiinfos = self.mdui_uiinfo(entity_id)
+        cls = classnames["mdui_uiinfo_logo"]
+        elements = self._lookup_elements_by_cls(uiinfos, cls)
+        values = (
+            element
+            for element in elements
+            if width is None or element.get("width") == width
+            if height is None or element.get("height") == height
+        )
+        return values
+
     def bindings(self, entity_id, typ, service):
         for _md in self.metadata.values():
             if entity_id in _md.items():
