@@ -9,6 +9,7 @@ from saml2.extension import mdui
 from saml2.extension import idpdisc
 from saml2.extension import shibmd
 from saml2.extension import mdattr
+from saml2.extension import sp_type
 from saml2.saml import NAME_FORMAT_URI
 from saml2.saml import AttributeValue
 from saml2.saml import Attribute
@@ -151,46 +152,17 @@ def do_organization_info(ava):
     return org
 
 
-def do_contact_person_info(lava):
-    """ Creates a ContactPerson instance from configuration information"""
+def do_contact_person_info(ava):
+    """Create a ContactPerson instance from configuration information."""
+    cper = md.ContactPerson()
+    cper.loadd(ava)
+    if not cper.contact_type:
+        cper.contact_type = "technical"
+    return cper
 
-    cps = []
-    if lava is None:
-        return cps
-
-    contact_person = md.ContactPerson
-    for ava in lava:
-        cper = md.ContactPerson()
-        for (key, classpec) in contact_person.c_children.values():
-            try:
-                value = ava[key]
-                data = []
-                if isinstance(classpec, list):
-                    # What if value is not a list ?
-                    if isinstance(value, six.string_types):
-                        data = [classpec[0](text=value)]
-                    else:
-                        for val in value:
-                            data.append(classpec[0](text=val))
-                else:
-                    data = classpec(text=value)
-                setattr(cper, key, data)
-            except KeyError:
-                pass
-        for (prop, classpec, _) in contact_person.c_attributes.values():
-            try:
-                # should do a check for valid value
-                setattr(cper, prop, ava[prop])
-            except KeyError:
-                pass
-
-        # ContactType must have a value
-        typ = getattr(cper, "contact_type")
-        if not typ:
-            setattr(cper, "contact_type", "technical")
-
-        cps.append(cper)
 
+def do_contact_persons_info(lava):
+    cps = [do_contact_person_info(ava) for ava in lava]
     return cps
 
 
@@ -695,20 +667,33 @@ def do_pdp_descriptor(conf, cert=None, enc_cert=None):
     return pdp
 
 
+def _add_attr_to_entity_attributes(extensions, attribute):
+    try:
+        entity_attributes = next(
+            el
+            for el in extensions.extension_elements
+            if el.tag == mdattr.EntityAttributes.c_tag
+        )
+    except StopIteration:
+        entity_attributes = mdattr.EntityAttributes(attribute=attribute)
+        extensions.add_extension_element(entity_attributes)
+    else:
+        entity_attributes.children.append(attribute)
+
+
 def entity_descriptor(confd):
     mycert = None
     enc_cert = None
     if confd.cert_file is not None:
         mycert = []
-        mycert.append("".join(open(confd.cert_file).readlines()[1:-1]))
+        mycert.append("".join(read_cert(confd.cert_file)))
         if confd.additional_cert_files is not None:
             for _cert_file in confd.additional_cert_files:
-                mycert.append("".join(open(_cert_file).readlines()[1:-1]))
+                mycert.append("".join(read_cert(_cert_file)))
     if confd.encryption_keypairs is not None:
         enc_cert = []
         for _encryption in confd.encryption_keypairs:
-            enc_cert.append(
-                "".join(open(_encryption["cert_file"]).readlines()[1:-1]))
+            enc_cert.append("".join(read_cert(_encryption["cert_file"])))
 
     entd = md.EntityDescriptor()
     entd.entity_id = confd.entityid
@@ -719,21 +704,49 @@ def entity_descriptor(confd):
     if confd.organization is not None:
         entd.organization = do_organization_info(confd.organization)
     if confd.contact_person is not None:
-        entd.contact_person = do_contact_person_info(confd.contact_person)
+        entd.contact_person = do_contact_persons_info(confd.contact_person)
+
+    if confd.assurance_certification:
+        if not entd.extensions:
+            entd.extensions = md.Extensions()
+        ava = [AttributeValue(text=c) for c in confd.assurance_certification]
+        attr = Attribute(
+            attribute_value=ava,
+            name="urn:oasis:names:tc:SAML:attribute:assurance-certification",
+        )
+        _add_attr_to_entity_attributes(entd.extensions, attr)
 
     if confd.entity_category:
-        entd.extensions = md.Extensions()
+        if not entd.extensions:
+            entd.extensions = md.Extensions()
         ava = [AttributeValue(text=c) for c in confd.entity_category]
-        attr = Attribute(attribute_value=ava,
-                         name="http://macedir.org/entity-category")
-        item = mdattr.EntityAttributes(attribute=attr)
-        entd.extensions.add_extension_element(item)
+        attr = Attribute(
+            attribute_value=ava, name="http://macedir.org/entity-category"
+        )
+        _add_attr_to_entity_attributes(entd.extensions, attr)
+
+    if confd.entity_category_support:
+        if not entd.extensions:
+            entd.extensions = md.Extensions()
+        ava = [AttributeValue(text=c) for c in confd.entity_category_support]
+        attr = Attribute(
+            attribute_value=ava, name="http://macedir.org/entity-category-support"
+        )
+        _add_attr_to_entity_attributes(entd.extensions, attr)
 
     for item in algorithm_support_in_metadata(confd.xmlsec_binary):
         if not entd.extensions:
             entd.extensions = md.Extensions()
         entd.extensions.add_extension_element(item)
 
+    conf_sp_type = confd.getattr('sp_type', 'sp')
+    conf_sp_type_in_md = confd.getattr('sp_type_in_metadata', 'sp')
+    if conf_sp_type and conf_sp_type_in_md is True:
+        if not entd.extensions:
+            entd.extensions = md.Extensions()
+        item = sp_type.SPType(text=conf_sp_type)
+        entd.extensions.add_extension_element(item)
+
     serves = confd.serves
     if not serves:
         raise SAMLError(
@@ -760,7 +773,8 @@ def entity_descriptor(confd):
     return entd
 
 
-def entities_descriptor(eds, valid_for, name, ident, sign, secc, sign_alg=None, digest_alg=None):
+def entities_descriptor(eds, valid_for, name, ident, sign, secc, sign_alg=None,
+                        digest_alg=None):
     entities = md.EntitiesDescriptor(entity_descriptor=eds)
     if valid_for:
         entities.valid_until = in_a_while(hours=valid_for)
@@ -781,7 +795,9 @@ def entities_descriptor(eds, valid_for, name, ident, sign, secc, sign_alg=None,
             raise SAMLError("If you want to do signing you should define " +
                             "where your public key are")
 
-        entities.signature = pre_signature_part(ident, secc.my_cert, 1, sign_alg=sign_alg, digest_alg=digest_alg)
+        entities.signature = pre_signature_part(ident, secc.my_cert, 1,
+                                                sign_alg=sign_alg,
+                                                digest_alg=digest_alg)
         entities.id = ident
         xmldoc = secc.sign_statement("%s" % entities, class_name(entities))
         entities = md.entities_descriptor_from_string(xmldoc)
@@ -808,3 +824,9 @@ def sign_entity_descriptor(edesc, ident, secc, sign_alg=None, digest_alg=None):
     xmldoc = secc.sign_statement("%s" % edesc, class_name(edesc))
     edesc = md.entity_descriptor_from_string(xmldoc)
     return edesc, xmldoc
+
+
+def read_cert(path):
+    with open(path) as fp:
+        lines = fp.readlines()
+    return lines[1:-1]
