@@ -1,6 +1,5 @@
 # coding=utf-8
-import pytest
-
+from saml2.argtree import add_path
 from saml2.authn_context import pword
 from saml2.mdie import to_dict
 from saml2 import md, assertion
@@ -20,20 +19,19 @@ from saml2.s_utils import MissingValue
 from saml2 import attribute_converter
 from saml2.attribute_converter import ac_factory, AttributeConverterNOOP
 
-from py.test import raises
+from pytest import raises
 
 from saml2.extension import mdui
 from saml2.extension import idpdisc
 from saml2.extension import dri
 from saml2.extension import mdattr
-from saml2.extension import ui
 from saml2 import saml
 from saml2 import xmldsig
 from saml2 import xmlenc
 
 from pathutils import full_path
 
-ONTS = [saml, mdui, mdattr, dri, ui, idpdisc, md, xmldsig, xmlenc]
+ONTS = [saml, mdui, mdattr, dri, idpdisc, md, xmldsig, xmlenc]
 
 
 def _eq(l1, l2):
@@ -63,7 +61,7 @@ def test_filter_on_attributes_0():
     required = [a]
     ava = {"serialNumber": ["12345"]}
 
-    ava = filter_on_attributes(ava, required)
+    ava = filter_on_attributes(ava, required, acs=ac_factory())
     assert list(ava.keys()) == ["serialNumber"]
     assert ava["serialNumber"] == ["12345"]
 
@@ -75,20 +73,36 @@ def test_filter_on_attributes_1():
     required = [a]
     ava = {"serialNumber": ["12345"], "givenName": ["Lars"]}
 
-    ava = filter_on_attributes(ava, required)
+    ava = filter_on_attributes(ava, required, acs=ac_factory())
     assert list(ava.keys()) == ["serialNumber"]
     assert ava["serialNumber"] == ["12345"]
 
 
+def test_filter_on_attributes_2():
+
+    a = to_dict(Attribute(friendly_name="surName",name="urn:oid:2.5.4.4",
+                          name_format=NAME_FORMAT_URI), ONTS)
+    required = [a]
+    ava = {"sn":["kakavas"]}
+
+    ava = filter_on_attributes(ava,required,acs=ac_factory())
+    assert list(ava.keys()) == ['sn']
+    assert ava["sn"] == ["kakavas"]
+
+
 def test_filter_on_attributes_without_friendly_name():
-    ava = {"eduPersonTargetedID": "test@example.com", "eduPersonAffiliation": "test",
+    ava = {"eduPersonTargetedID": "test@example.com",
+           "eduPersonAffiliation": "test",
            "extra": "foo"}
     eptid = to_dict(
-            Attribute(name="urn:oid:1.3.6.1.4.1.5923.1.1.1.10", name_format=NAME_FORMAT_URI), ONTS)
+        Attribute(name="urn:oid:1.3.6.1.4.1.5923.1.1.1.10",
+                  name_format=NAME_FORMAT_URI), ONTS)
     ep_affiliation = to_dict(
-            Attribute(name="urn:oid:1.3.6.1.4.1.5923.1.1.1.1", name_format=NAME_FORMAT_URI), ONTS)
+        Attribute(name="urn:oid:1.3.6.1.4.1.5923.1.1.1.1",
+                  name_format=NAME_FORMAT_URI), ONTS)
 
-    restricted_ava = filter_on_attributes(ava, required=[eptid], optional=[ep_affiliation],
+    restricted_ava = filter_on_attributes(ava, required=[eptid],
+                                          optional=[ep_affiliation],
                                           acs=ac_factory())
     assert restricted_ava == {"eduPersonTargetedID": "test@example.com",
                               "eduPersonAffiliation": "test"}
@@ -97,18 +111,31 @@ def test_filter_on_attributes_without_friendly_name():
 def test_filter_on_attributes_with_missing_required_attribute():
     ava = {"extra": "foo"}
     eptid = to_dict(Attribute(
-            friendly_name="eduPersonTargetedID", name="urn:oid:1.3.6.1.4.1.5923.1.1.1.10",
-            name_format=NAME_FORMAT_URI), ONTS)
-    with pytest.raises(MissingValue):
-        filter_on_attributes(ava, required=[eptid])
+        friendly_name="eduPersonTargetedID",
+        name="urn:oid:1.3.6.1.4.1.5923.1.1.1.10",
+        name_format=NAME_FORMAT_URI), ONTS)
+    with raises(MissingValue):
+        filter_on_attributes(ava, required=[eptid], acs=ac_factory())
 
 
 def test_filter_on_attributes_with_missing_optional_attribute():
     ava = {"extra": "foo"}
     eptid = to_dict(Attribute(
-            friendly_name="eduPersonTargetedID", name="urn:oid:1.3.6.1.4.1.5923.1.1.1.10",
-            name_format=NAME_FORMAT_URI), ONTS)
-    assert filter_on_attributes(ava, optional=[eptid]) == {}
+        friendly_name="eduPersonTargetedID",
+        name="urn:oid:1.3.6.1.4.1.5923.1.1.1.10",
+        name_format=NAME_FORMAT_URI), ONTS)
+    assert filter_on_attributes(ava, optional=[eptid], acs=ac_factory()) == {}
+
+
+def test_filter_on_attributes_with_missing_name_format():
+    ava = {"eduPersonTargetedID": "test@example.com",
+           "eduPersonAffiliation": "test",
+           "extra": "foo"}
+    eptid = to_dict(Attribute(friendly_name="eduPersonTargetedID",
+                              name="urn:myown:eptid",
+                              name_format=''), ONTS)
+    ava = filter_on_attributes(ava, optional=[eptid], acs=ac_factory())
+    assert ava['eduPersonTargetedID'] == "test@example.com"
 
 
 # ----------------------------------------------------------------------
@@ -129,6 +156,7 @@ def test_lifetime_1():
         }}
 
     r = Policy(conf)
+
     assert r is not None
 
     assert r.get_lifetime("urn:mace:umu.se:saml:roland:sp") == {"minutes": 5}
@@ -196,25 +224,22 @@ def test_ava_filter_2():
             "lifetime": {"minutes": 5},
             "attribute_restrictions": {
                 "givenName": None,
-                "surName": None,
+                "sn": None,
                 "mail": [".*@.*\.umu\.se"],
             }
         }}
 
     policy = Policy(conf)
 
-    ava = {"givenName": "Derek",
-           "surName": "Jeter",
-           "mail": "derek@example.com"}
+    ava = {"givenName": "Derek", "sn": "Jeter", "mail": "derek@example.com"}
 
     # mail removed because it doesn't match the regular expression
     _ava = policy.filter(ava, 'urn:mace:umu.se:saml:roland:sp', None, [mail],
                          [gn, sn])
 
-    assert _eq(sorted(list(_ava.keys())), ["givenName", "surName"])
+    assert _eq(sorted(list(_ava.keys())), ["givenName", 'sn'])
 
-    ava = {"givenName": "Derek",
-           "surName": "Jeter"}
+    ava = {"givenName": "Derek", "sn": "Jeter"}
 
     # it wasn't there to begin with
     try:
@@ -413,94 +438,96 @@ def test_filter_values_req_2():
     required = [a1, a2]
     ava = {"serialNumber": ["12345"], "givenName": ["Lars"]}
 
-    raises(MissingValue, filter_on_attributes, ava, required)
+    with raises(MissingValue):
+        filter_on_attributes(ava, required, acs=ac_factory())
 
 
 def test_filter_values_req_3():
     a = to_dict(
-            Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                      friendly_name="serialNumber",
-                      attribute_value=[AttributeValue(text="12345")]), ONTS)
+        Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
+                  friendly_name="serialNumber",
+                  attribute_value=[AttributeValue(text="12345")]), ONTS)
 
     required = [a]
     ava = {"serialNumber": ["12345"]}
 
-    ava = filter_on_attributes(ava, required)
+    ava = filter_on_attributes(ava, required, acs=ac_factory())
     assert list(ava.keys()) == ["serialNumber"]
     assert ava["serialNumber"] == ["12345"]
 
 
 def test_filter_values_req_4():
     a = to_dict(
-            Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                      friendly_name="serialNumber",
-                      attribute_value=[AttributeValue(text="54321")]), ONTS)
+        Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
+                  friendly_name="serialNumber",
+                  attribute_value=[AttributeValue(text="54321")]), ONTS)
 
     required = [a]
     ava = {"serialNumber": ["12345"]}
 
-    raises(MissingValue, filter_on_attributes, ava, required)
+    with raises(MissingValue):
+        filter_on_attributes(ava, required, acs=ac_factory())
 
 
 def test_filter_values_req_5():
     a = to_dict(
-            Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                      friendly_name="serialNumber",
-                      attribute_value=[AttributeValue(text="12345")]), ONTS)
+        Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
+                  friendly_name="serialNumber",
+                  attribute_value=[AttributeValue(text="12345")]), ONTS)
 
     required = [a]
     ava = {"serialNumber": ["12345", "54321"]}
 
-    ava = filter_on_attributes(ava, required)
+    ava = filter_on_attributes(ava, required, acs=ac_factory())
     assert list(ava.keys()) == ["serialNumber"]
     assert ava["serialNumber"] == ["12345"]
 
 
 def test_filter_values_req_6():
     a = to_dict(
-            Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                      friendly_name="serialNumber",
-                      attribute_value=[AttributeValue(text="54321")]), ONTS)
+        Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
+                  friendly_name="serialNumber",
+                  attribute_value=[AttributeValue(text="54321")]), ONTS)
 
     required = [a]
     ava = {"serialNumber": ["12345", "54321"]}
 
-    ava = filter_on_attributes(ava, required)
+    ava = filter_on_attributes(ava, required, acs=ac_factory())
     assert list(ava.keys()) == ["serialNumber"]
     assert ava["serialNumber"] == ["54321"]
 
 
 def test_filter_values_req_opt_0():
     r = to_dict(
-            Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                      friendly_name="serialNumber",
-                      attribute_value=[AttributeValue(text="54321")]), ONTS)
+        Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
+                  friendly_name="serialNumber",
+                  attribute_value=[AttributeValue(text="54321")]), ONTS)
     o = to_dict(
-            Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                      friendly_name="serialNumber",
-                      attribute_value=[AttributeValue(text="12345")]), ONTS)
+        Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
+                  friendly_name="serialNumber",
+                  attribute_value=[AttributeValue(text="12345")]), ONTS)
 
     ava = {"serialNumber": ["12345", "54321"]}
 
-    ava = filter_on_attributes(ava, [r], [o])
+    ava = filter_on_attributes(ava, [r], [o], acs=ac_factory())
     assert list(ava.keys()) == ["serialNumber"]
     assert _eq(ava["serialNumber"], ["12345", "54321"])
 
 
 def test_filter_values_req_opt_1():
     r = to_dict(
-            Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                      friendly_name="serialNumber",
-                      attribute_value=[AttributeValue(text="54321")]), ONTS)
+        Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
+                  friendly_name="serialNumber",
+                  attribute_value=[AttributeValue(text="54321")]), ONTS)
     o = to_dict(
-            Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                      friendly_name="serialNumber",
-                      attribute_value=[AttributeValue(text="12345"),
-                                       AttributeValue(text="abcd0")]), ONTS)
+        Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
+                  friendly_name="serialNumber",
+                  attribute_value=[AttributeValue(text="12345"),
+                                   AttributeValue(text="abcd0")]), ONTS)
 
     ava = {"serialNumber": ["12345", "54321"]}
 
-    ava = filter_on_attributes(ava, [r], [o])
+    ava = filter_on_attributes(ava, [r], [o], acs=ac_factory())
     assert list(ava.keys()) == ["serialNumber"]
     assert _eq(ava["serialNumber"], ["12345", "54321"])
 
@@ -508,35 +535,36 @@ def test_filter_values_req_opt_1():
 def test_filter_values_req_opt_2():
     r = [
         to_dict(
-                Attribute(
-                        friendly_name="surName",
-                        name="urn:oid:2.5.4.4",
-                        name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
-                ONTS),
+            Attribute(
+                friendly_name="surName",
+                name="urn:oid:2.5.4.4",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
+            ONTS),
         to_dict(
-                Attribute(
-                        friendly_name="givenName",
-                        name="urn:oid:2.5.4.42",
-                        name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
-                ONTS),
+            Attribute(
+                friendly_name="givenName",
+                name="urn:oid:2.5.4.42",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
+            ONTS),
         to_dict(
-                Attribute(
-                        friendly_name="mail",
-                        name="urn:oid:0.9.2342.19200300.100.1.3",
-                        name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
-                ONTS)]
+            Attribute(
+                friendly_name="mail",
+                name="urn:oid:0.9.2342.19200300.100.1.3",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
+            ONTS)]
     o = [
         to_dict(
-                Attribute(
-                        friendly_name="title",
-                        name="urn:oid:2.5.4.12",
-                        name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
-                ONTS)]
+            Attribute(
+                friendly_name="title",
+                name="urn:oid:2.5.4.12",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
+            ONTS)]
 
     ava = {"surname": ["Hedberg"], "givenName": ["Roland"],
            "eduPersonAffiliation": ["staff"], "uid": ["rohe0002"]}
 
-    raises(MissingValue, "filter_on_attributes(ava, r, o)")
+    with raises(MissingValue):
+        filter_on_attributes(ava, r, o, acs=ac_factory())
 
 
 # ---------------------------------------------------------------------------
@@ -545,18 +573,18 @@ def test_filter_values_req_opt_2():
 def test_filter_values_req_opt_4():
     r = [
         Attribute(
-                friendly_name="surName",
-                name="urn:oid:2.5.4.4",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
+            friendly_name="surName",
+            name="urn:oid:2.5.4.4",
+            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
         Attribute(
-                friendly_name="givenName",
-                name="urn:oid:2.5.4.42",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
+            friendly_name="givenName",
+            name="urn:oid:2.5.4.42",
+            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
     o = [
         Attribute(
-                friendly_name="title",
-                name="urn:oid:2.5.4.12",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
+            friendly_name="title",
+            name="urn:oid:2.5.4.12",
+            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
 
     acs = attribute_converter.ac_factory(full_path("attributemaps"))
 
@@ -577,15 +605,15 @@ def test_filter_values_req_opt_4():
 
 def test_filter_ava_0():
     policy = Policy(
-            {
-                "default": {
-                    "lifetime": {"minutes": 15},
-                    "attribute_restrictions": None  # means all I have
-                },
-                "urn:mace:example.com:saml:roland:sp": {
-                    "lifetime": {"minutes": 5},
-                }
+        {
+            "default": {
+                "lifetime": {"minutes": 15},
+                "attribute_restrictions": None  # means all I have
+            },
+            "urn:mace:example.com:saml:roland:sp": {
+                "lifetime": {"minutes": 5},
             }
+        }
     )
 
     ava = {"givenName": ["Derek"], "surName": ["Jeter"],
@@ -701,33 +729,33 @@ def test_filter_ava_4():
 def test_req_opt():
     req = [
         to_dict(
-                md.RequestedAttribute(
-                        friendly_name="surname", name="urn:oid:2.5.4.4",
-                        name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
-                        is_required="true"), ONTS),
+            md.RequestedAttribute(
+                friendly_name="surname", name="urn:oid:2.5.4.4",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
+                is_required="true"), ONTS),
         to_dict(
-                md.RequestedAttribute(
-                        friendly_name="givenname",
-                        name="urn:oid:2.5.4.42",
-                        name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
-                        is_required="true"), ONTS),
+            md.RequestedAttribute(
+                friendly_name="givenname",
+                name="urn:oid:2.5.4.42",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
+                is_required="true"), ONTS),
         to_dict(
-                md.RequestedAttribute(
-                        friendly_name="edupersonaffiliation",
-                        name="urn:oid:1.3.6.1.4.1.5923.1.1.1.1",
-                        name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
-                        is_required="true"), ONTS)]
+            md.RequestedAttribute(
+                friendly_name="edupersonaffiliation",
+                name="urn:oid:1.3.6.1.4.1.5923.1.1.1.1",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
+                is_required="true"), ONTS)]
 
     opt = [
         to_dict(
-                md.RequestedAttribute(
-                        friendly_name="title",
-                        name="urn:oid:2.5.4.12",
-                        name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
-                        is_required="false"), ONTS)]
+            md.RequestedAttribute(
+                friendly_name="title",
+                name="urn:oid:2.5.4.12",
+                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
+                is_required="false"), ONTS)]
 
     policy = Policy()
-    ava = {'givenname': 'Roland', 'surname': 'Hedberg',
+    ava = {'givenname': 'Roland', 'sn': 'Hedberg',
            'uid': 'rohe0002', 'edupersonaffiliation': 'staff'}
 
     sp_entity_id = "urn:mace:example.com:saml:curt:sp"
@@ -738,18 +766,18 @@ def test_req_opt():
 def test_filter_on_wire_representation_1():
     r = [
         Attribute(
-                friendly_name="surName",
-                name="urn:oid:2.5.4.4",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
+            friendly_name="surName",
+            name="urn:oid:2.5.4.4",
+            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
         Attribute(
-                friendly_name="givenName",
-                name="urn:oid:2.5.4.42",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
+            friendly_name="givenName",
+            name="urn:oid:2.5.4.42",
+            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
     o = [
         Attribute(
-                friendly_name="title",
-                name="urn:oid:2.5.4.12",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
+            friendly_name="title",
+            name="urn:oid:2.5.4.12",
+            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
 
     acs = attribute_converter.ac_factory(full_path("attributemaps"))
 
@@ -763,18 +791,18 @@ def test_filter_on_wire_representation_1():
 def test_filter_on_wire_representation_2():
     r = [
         Attribute(
-                friendly_name="surName",
-                name="urn:oid:2.5.4.4",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
+            friendly_name="surName",
+            name="urn:oid:2.5.4.4",
+            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri"),
         Attribute(
-                friendly_name="givenName",
-                name="urn:oid:2.5.4.42",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
+            friendly_name="givenName",
+            name="urn:oid:2.5.4.42",
+            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
     o = [
         Attribute(
-                friendly_name="title",
-                name="urn:oid:2.5.4.12",
-                name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
+            friendly_name="title",
+            name="urn:oid:2.5.4.12",
+            name_format="urn:oasis:names:tc:SAML:2.0:attrname-format:uri")]
 
     acs = attribute_converter.ac_factory(full_path("attributemaps"))
 
@@ -804,10 +832,21 @@ def test_assertion_with_noop_attribute_conv():
     })
     name_id = NameID(format=NAMEID_FORMAT_TRANSIENT, text="foobar")
     issuer = Issuer(text="entityid", format=NAMEID_FORMAT_ENTITY)
-    msg = ast.construct("sp_entity_id", "in_response_to", "consumer_url",
-                        name_id, [AttributeConverterNOOP(NAME_FORMAT_URI)],
-                        policy, issuer=issuer, authn_decl=ACD,
-                        authn_auth="authn_authn")
+
+    farg = add_path(
+        {},
+        ['subject', 'subject_confirmation', 'method', saml.SCM_BEARER])
+    add_path(
+        farg['subject']['subject_confirmation'],
+        ['subject_confirmation_data', 'in_response_to', 'in_response_to'])
+    add_path(
+        farg['subject']['subject_confirmation'],
+        ['subject_confirmation_data', 'recipient', 'consumer_url'])
+
+    msg = ast.construct(
+        "sp_entity_id", [AttributeConverterNOOP(NAME_FORMAT_URI)], policy,
+        issuer=issuer, farg=farg, authn_decl=ACD, name_id=name_id,
+        authn_auth="authn_authn")
 
     print(msg)
     for attr in msg.attribute_statement[0].attribute:
@@ -852,10 +891,20 @@ def test_assertion_with_zero_attributes():
     })
     name_id = NameID(format=NAMEID_FORMAT_TRANSIENT, text="foobar")
     issuer = Issuer(text="entityid", format=NAMEID_FORMAT_ENTITY)
-    msg = ast.construct("sp_entity_id", "in_response_to", "consumer_url",
-                        name_id, [AttributeConverterNOOP(NAME_FORMAT_URI)],
-                        policy, issuer=issuer, authn_decl=ACD,
-                        authn_auth="authn_authn")
+    farg = add_path(
+        {},
+        ['subject', 'subject_confirmation', 'method', saml.SCM_BEARER])
+    add_path(
+        farg['subject']['subject_confirmation'],
+        ['subject_confirmation_data', 'in_response_to', 'in_response_to'])
+    add_path(
+        farg['subject']['subject_confirmation'],
+        ['subject_confirmation_data', 'recipient', 'consumer_url'])
+
+    msg = ast.construct(
+        "sp_entity_id", [AttributeConverterNOOP(NAME_FORMAT_URI)], policy,
+        issuer=issuer, authn_decl=ACD, authn_auth="authn_authn",
+        name_id=name_id, farg=farg)
 
     print(msg)
     assert msg.attribute_statement == []
@@ -873,11 +922,21 @@ def test_assertion_with_authn_instant():
     })
     name_id = NameID(format=NAMEID_FORMAT_TRANSIENT, text="foobar")
     issuer = Issuer(text="entityid", format=NAMEID_FORMAT_ENTITY)
-    msg = ast.construct("sp_entity_id", "in_response_to", "consumer_url",
-                        name_id, [AttributeConverterNOOP(NAME_FORMAT_URI)],
-                        policy, issuer=issuer, authn_decl=ACD,
-                        authn_auth="authn_authn",
-                        authn_instant=1234567890)
+
+    farg = add_path(
+        {},
+        ['subject', 'subject_confirmation', 'method', saml.SCM_BEARER])
+    add_path(
+        farg['subject']['subject_confirmation'],
+        ['subject_confirmation_data', 'in_response_to', 'in_response_to'])
+    add_path(
+        farg['subject']['subject_confirmation'],
+        ['subject_confirmation_data', 'recipient', 'consumer_url'])
+
+    msg = ast.construct(
+        "sp_entity_id", [AttributeConverterNOOP(NAME_FORMAT_URI)], policy,
+        issuer=issuer, authn_decl=ACD, authn_auth="authn_authn",
+        authn_instant=1234567890, name_id=name_id, farg=farg)
 
     print(msg)
     assert msg.authn_statement[0].authn_instant == "2009-02-13T23:31:30Z"
@@ -885,3 +944,4 @@ def test_assertion_with_authn_instant():
 
 if __name__ == "__main__":
     test_assertion_2()
+
