@@ -2,15 +2,18 @@
 # -*- coding: utf-8 -*-
 import datetime
 import re
-from six.moves.urllib.parse import quote_plus
-#from future.backports.urllib.parse import quote_plus
+from collections import OrderedDict
+
+from six.moves.urllib import parse
+
 from saml2.config import Config
-from saml2.mdstore import MetadataStore
+from saml2.mdstore import MetadataStore, MetaDataExtern
 from saml2.mdstore import MetaDataMDX
 from saml2.mdstore import SAML_METADATA_CONTENT_TYPE
 from saml2.mdstore import destinations
 from saml2.mdstore import name
 from saml2 import sigver
+from saml2.httpbase import HTTPBase
 from saml2 import BINDING_SOAP
 from saml2 import BINDING_HTTP_REDIRECT
 from saml2 import BINDING_HTTP_POST
@@ -296,7 +299,7 @@ def test_mdx_service():
     entity_id = "http://xenosmilus.umdc.umu.se/simplesaml/saml2/idp/metadata.php"
 
     url = "http://mdx.example.com/entities/{}".format(
-        quote_plus(MetaDataMDX.sha1_entity_transform(entity_id)))
+        parse.quote_plus(MetaDataMDX.sha1_entity_transform(entity_id)))
     responses.add(responses.GET, url, body=TEST_METADATA_STRING, status=200,
                   content_type=SAML_METADATA_CONTENT_TYPE)
 
@@ -312,7 +315,7 @@ def test_mdx_single_sign_on_service():
     entity_id = "http://xenosmilus.umdc.umu.se/simplesaml/saml2/idp/metadata.php"
 
     url = "http://mdx.example.com/entities/{}".format(
-        quote_plus(MetaDataMDX.sha1_entity_transform(entity_id)))
+        parse.quote_plus(MetaDataMDX.sha1_entity_transform(entity_id)))
     responses.add(responses.GET, url, body=TEST_METADATA_STRING, status=200,
                   content_type=SAML_METADATA_CONTENT_TYPE)
 
@@ -372,7 +375,8 @@ def test_load_extern_incommon():
 
 def test_load_local():
     # string representation of XML idp definition
-    idp_metadata = open(full_path("metadata.xml")).read()
+    with open(full_path("metadata.xml")) as fp:
+        idp_metadata = fp.read()
 
     saml_config = Config()
 
@@ -383,6 +387,14 @@ def test_load_local():
     assert cfg
 
 
+def test_load_remote_encoding():
+    crypto = sigver._get_xmlsec_cryptobackend()
+    sc = sigver.SecurityContext(crypto, key_type="", cert_type="")
+    httpc = HTTPBase()
+    mds = MetaDataExtern(ATTRCONV, 'http://metadata.aai.switch.ch/metadata.aaitest.xml', sc, full_path('SWITCHaaiRootCA.crt.pem'), httpc)
+    mds.load()
+
+
 def test_load_string():
     sec_config.xmlsec_binary = sigver.get_xmlsec_binary(["/opt/local/bin"])
     mds = MetadataStore(ATTRCONV, sec_config,
@@ -447,11 +459,23 @@ def test_get_certs_from_metadata_without_keydescriptor():
 
     assert len(certs) == 0
 
+
 def test_metadata_extension_algsupport():
     mds = MetadataStore(ATTRCONV, None)
     mds.imp(METADATACONF["12"])
     mdf = mds.metadata[full_path("uu.xml")]
     assert mds
 
+
+def test_extension():
+    mds = MetadataStore(ATTRCONV, None)
+    # use ordered dict to force expected entity to be last
+    metadata = OrderedDict()
+    metadata["1"] = {"entity1": {}}
+    metadata["2"] = {"entity2": {"idpsso_descriptor": [{"extensions": {"extension_elements": [{"__class__": "test"}]}}]}}
+    mds.metadata = metadata
+    assert mds.extension("entity2", "idpsso_descriptor", "test")
+
+
 if __name__ == "__main__":
     test_metadata_extension_algsupport()
