@@ -1,4 +1,4 @@
-import xmldsig as ds
+from saml2 import xmldsig as ds
 from saml2 import config
 from saml2 import extension_elements_to_elements
 from saml2 import element_to_extension_element
@@ -35,7 +35,7 @@ req_id, req = client.create_authn_request(
     extensions=extensions)
 
 
-print req
+print(req)
 
 # Get a certificate from an authn request
 
@@ -48,5 +48,5 @@ _elem = extension_elements_to_elements(parsed.extensions.extension_elements,
 
 assert len(_elem) == 1
 _spcertenc = _elem[0]
-_cert = _spcertenc.x509_data[0].x509_certificate.text
+_cert = _spcertenc.key_info[0].x509_data[0].x509_certificate.text
 assert cert == _cert
